"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const web_ui_1 = __importDefault(require("@hypernetlabs/web-ui"));
const rxjs_1 = require("rxjs");
const HypernetIFrameProxy_1 = __importDefault(require("../../implementations/proxy/HypernetIFrameProxy"));
class HypernetWebIntegration {
    constructor(iframeURL) {
        this.iframeURL = "http://localhost:5020"; // TODO: This should eventually be mainnet release
        this.getReadyTimeout = 15 * 1000;
        this.getReadyResolved = false;
        this.selectedStateChannel = {};
        this.iframeURL = iframeURL || this.iframeURL;
        // Create a proxy connection to the iframe
        this.core = new HypernetIFrameProxy_1.default(this._prepareIFrameContainer(), this.iframeURL, "hypernet-core-iframe");
        this.UIData = {
            onSelectedStateChannelChanged: new rxjs_1.Subject(),
            onVotesDelegated: new rxjs_1.Subject(),
            getSelectedStateChannel: () => this.selectedStateChannel,
        };
        this.UIData.onSelectedStateChannelChanged.subscribe((stateChannel) => {
            this.selectedStateChannel = stateChannel;
        });
        if (window.hypernetWebUIInstance) {
            this.webUIClient = window.hypernetWebUIInstance;
        }
        else {
            this.webUIClient = new web_ui_1.default(this.core, this.UIData);
        }
        this.core.onGatewayIFrameDisplayRequested.subscribe((gatewayUrl) => {
            this.currentGatewayUrl = gatewayUrl;
        });
        this.core.onPrivateCredentialsRequested.subscribe(() => {
            //this.webUIClient.renderPrivateKeysModal();
            this.webUIClient.renderMetamaskWarningModal();
        });
        this.core.onWalletConnectOptionsDisplayRequested.subscribe(() => {
            this.webUIClient.renderWalletConnectWidget({ showInModal: true });
        });
    }
    // wait for the core to be intialized
    getReady() {
        // Wait getReadyTimeout and show timeout guid if getReady hasn't resolved yet
        /* setTimeout(() => {
          if (this.getReadyResolved === false) {
            this.webUIClient.renderWarningAlertModal(
              "Timeout exceeded while initializing Hypernet Protocol!",
            );
          }
        }, this.getReadyTimeout); */
        if (this.getReadyResult != null) {
            return this.getReadyResult;
        }
        this.getReadyResult = this.core
            .activate()
            .andThen(() => this.core.initialize())
            .map(() => {
            // This is for web ui to use if there is no core instance passed in web ui constructor
            window.hypernetCoreInstance = this.core;
            this.getReadyResolved = true;
            return this.core;
        })
            .mapErr((err) => {
            this.getReadyResolved = true;
            this.webUIClient.renderWarningAlertModal(`an error occurred during initialization of hypernet core${(err === null || err === void 0 ? void 0 : err.message) ? `: ${err === null || err === void 0 ? void 0 : err.message}` : "."}`);
            return new Error("Something went wrong!");
        });
        return this.getReadyResult;
    }
    // This class must be used as a singleton, this enforces that restriction.
    static getInstance() {
        if (HypernetWebIntegration.instance == null) {
            HypernetWebIntegration.instance = new HypernetWebIntegration();
        }
        return HypernetWebIntegration.instance;
    }
    displayGatewayIFrame(gatewayUrl) {
        this.core.displayGatewayIFrame(gatewayUrl);
    }
    closeGatewayIFrame(gatewayUrl) {
        this.core.closeGatewayIFrame(gatewayUrl);
    }
    _prepareIFrameContainer() {
        // Create a container element for the iframe proxy
        const iframeContainer = document.createElement("div");
        iframeContainer.id = "__hypernet-protocol-iframe-container__";
        // Add close modal icon to iframe container
        const closeButton = document.createElement("div");
        closeButton.id = "__hypernet-protocol-iframe-close-icon__";
        closeButton.innerHTML = `
      <img id="__hypernet-protocol-iframe-close-img__" src="https://res.cloudinary.com/dqueufbs7/image/upload/v1611371438/images/Close-512.png" width="20" />
    `;
        iframeContainer.appendChild(closeButton);
        // Add iframe modal style
        const style = document.createElement("style");
        style.appendChild(document.createTextNode(`
        iframe {
          position: absolute;
          display: none;
          border: none;
          width: 550px;
          height: 60%;
          min-height: 200px;
          background-color: white;
          top: 50%;
          left: 50%;
          box-shadow: 0px 4px 20px #000000;
          border-radius: 4px;
          transform: translate(-50%, -50%);
        }
        #__hypernet-protocol-iframe-container__ {
          position: fixed;
          display: none;
          top: 0;
          left: 0;
          height: 100%;
          width: 100%;
          background-color: rgba(0,0,0,0.6);
          z-index: 999999 !important;
        }
        #__hypernet-protocol-iframe-close-icon__ {
          z-index: 2;
          position: absolute;
          height: 60%;
          top: 50%;
          left: 50%;
          transform: translate(calc(-50% + 263px), -50%);
        }
        #__hypernet-protocol-iframe-close-img__{
          cursor: pointer;
        }
    `));
        document.head.appendChild(style);
        // Attach everything to the body
        document.body.appendChild(iframeContainer);
        const closeImg = document.getElementById("__hypernet-protocol-iframe-close-img__");
        closeImg === null || closeImg === void 0 ? void 0 : closeImg.addEventListener("click", (e) => {
            if (this.currentGatewayUrl != null) {
                this.core.closeGatewayIFrame(this.currentGatewayUrl);
                this.currentGatewayUrl = null;
            }
            iframeContainer.style.display = "none";
        }, false);
        return iframeContainer;
    }
}
exports.default = HypernetWebIntegration;
window.HypernetWebIntegration = HypernetWebIntegration;
//# sourceMappingURL=HypernetWebIntegration.js.map